;
; This file includes helpful functions to monitor the MIDI Rx/Tx activity
;
;    o MIDI_RXTX_NotifyRx:    to be called when a MIDI byte has been received
;    o MIDI_RXTX_NotifyTx:    to be called when a MIDI byte has been sent
;    o MIDI_RXTX_Handler:     to be called from USER_SR_Service_Prepare
;
; See the function headers for further details
;
; Settings which can be modified:
;
; Number of update cycles (* 1mS) the LEDs will stay active on a Rx/Tx event
#define MIDI_RXTX_LED_DELAY 15
; DOUT pin number of Rx LED
#define MIDI_RXTX_RX_LED    MIDI_MONITOR_RX_LED
; DOUT pin number of Tx LED
#define MIDI_RXTX_TX_LED    MIDI_MONITOR_TX_LED
;
; Following registers have to be located to free addresses in app_defines.h:
;
;MIDI_RXTX_RX_CTR	EQU	0x019
;MIDI_RXTX_TX_CTR	EQU	0x01a
;
; ==========================================================================
;
; Copyright (C) 1998-2003  Thorsten Klose (Thorsten.Klose@midibox.org)
;                          http://www.uCApps.de
; 
; ==========================================================================
; 
; This file is part of an example Application
;
; This application is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This application is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this application; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
; ==========================================================================


;; --------------------------------------------------------------------------
;;  FUNCTION: MIDI_RXTX_NotifyRx
;;  DESCRIPTION: this function has to be called from the USER_MIDI_NotifyRx
;;  hook when a MIDI event has been received to reload the RX counter
;; --------------------------------------------------------------------------
MIDI_RXTX_NotifyRx
	movlw	MIDI_RXTX_LED_DELAY
	SET_BSR	MIDI_RXTX_RX_CTR
	movwf	MIDI_RXTX_RX_CTR, BANKED
	return

;; --------------------------------------------------------------------------
;;  FUNCTION: MIDI_RXTX_NotifyTx
;;  DESCRIPTION: this function has to be called from the USER_MIDI_NotifyTx
;;  hook when a MIDI event will be transmitted to reload the TX counter
;; --------------------------------------------------------------------------
MIDI_RXTX_NotifyTx
	movlw	MIDI_RXTX_LED_DELAY
	SET_BSR	MIDI_RXTX_TX_CTR
	movwf	MIDI_RXTX_TX_CTR, BANKED
	return

;; --------------------------------------------------------------------------
;;  FUNCTION: MIDI_RXTX_Handler
;;  DESCRIPTION: this function has to be called from the USER_SR_ServicePrpeare
;;  hook, it decrements the Rx/Tx counters and sets the LEDs depending on the
;;  counter values
;; --------------------------------------------------------------------------
MIDI_RXTX_Handler
	;; Decrement Rx counter if != 0
	SET_BSR	MIDI_RXTX_RX_CTR
	movf	MIDI_RXTX_RX_CTR, W, BANKED
	skpz
	decf	MIDI_RXTX_RX_CTR, F, BANKED

	;; Decrement Tx counter if != 0
	SET_BSR	MIDI_RXTX_TX_CTR
	movf	MIDI_RXTX_TX_CTR, W, BANKED
	skpz
	decf	MIDI_RXTX_TX_CTR, F, BANKED


	;; 
	;; remove the code below if you don't want to use LEDs to
	;; indicate the counter state
	;; 

	;; set the Rx LED depending on counter state
	SET_BSR	MIDI_RXTX_RX_CTR
	movf	MIDI_RXTX_RX_CTR, W, BANKED
	skpz
	movlw	0x01
	movwf	MIOS_PARAMETER1
	movlw	MIDI_RXTX_RX_LED
	call	MIOS_DOUT_PinSet

	;; set the Tx LED depending on counter state
	SET_BSR	MIDI_RXTX_TX_CTR
	movf	MIDI_RXTX_TX_CTR, W, BANKED
	skpz
	movlw	0x01
	movwf	MIOS_PARAMETER1
	movlw	MIDI_RXTX_TX_LED
	call	MIOS_DOUT_PinSet

	return
